# flake8: noqa E501

import json
import logging
import tkinter as tk
from gettext import gettext as _
from sys import exit
from tkinter import messagebox, ttk

import cv2
import numpy as np
import torch
from easydict import EasyDict as edict
from PIL import Image

from ritm_annotation.cli.annotate.canvas import CanvasImage
from ritm_annotation.cli.annotate.controller import InteractiveController
from ritm_annotation.cli.annotate.wrappers import (
    BoundedNumericalEntry,
    FocusButton,
    FocusCheckButton,
    FocusHorizontalScale,
    FocusLabelFrame,
)
from ritm_annotation.inference.utils import find_checkpoint, load_is_model
from ritm_annotation.utils.misc import (
    get_default_weight,
    ignore_params_then_call,
)

logger = logging.getLogger(__name__)


class InteractiveDemoApp(ttk.Frame):
    def __init__(self, master, args, model, tasks_iterator, classes_first=True):
        logger.info("Initializing...")
        super().__init__(master)
        self.classes_first = classes_first
        self.tasks = list(tasks_iterator)
        self._current_task_idx = 0
        self.master = master
        master.title(
            _(
                "Reviving Iterative Training with Mask Guidance for Interactive Segmentation"
            )  # noqa: E501
        )
        master.withdraw()
        master.update_idletasks()
        x = (master.winfo_screenwidth() - master.winfo_reqwidth()) / 2
        y = (master.winfo_screenheight() - master.winfo_reqheight()) / 2
        master.geometry("+%d+%d" % (x, y))
        self.pack(fill="both", expand=True)

        self.brs_modes = [
            "NoBRS",
            "RGB-BRS",
            "DistMap-BRS",
            "f-BRS-A",
            "f-BRS-B",
            "f-BRS-C",
        ]
        self.limit_longest_size = args.limit_longest_size

        self.controller = InteractiveController(
            model,
            args.device,
            predictor_params={"brs_mode": "NoBRS"},
            update_image_callback=self._update_image,
        )

        self._init_state()
        self._add_menu()
        self._add_canvas()
        self._add_buttons()

        master.bind("<Escape>", lambda event: exit(0))
        master.bind(
            "j",
            lambda event: self.state["visualize_mask"].set(
                not self.state["visualize_mask"].get()
            ),
        )
        master.bind(
            "k",
            lambda event: self.state["visualize_blender_contrast"].set(
                not self.state["visualize_blender_contrast"].get()
            ),
        )
        master.bind("l", lambda event: self._save_mask_callback())
        master.bind("h", lambda event: self._undo_click())
        master.bind("m", lambda event: self._reset_last_object())
        master.bind("n", lambda event: self._load_image_callback())
        master.bind("b", lambda event: self._goto_previous_task())

        self.state["zoomin_params"]["skip_clicks"].trace(
            mode="w", callback=self._reset_predictor
        )
        self.state["zoomin_params"]["target_size"].trace(
            mode="w", callback=self._reset_predictor
        )
        self.state["zoomin_params"]["expansion_ratio"].trace(
            mode="w", callback=self._reset_predictor
        )
        self.state["predictor_params"]["net_clicks_limit"].trace(
            mode="w", callback=self._change_brs_mode
        )
        self.state["lbfgs_max_iters"].trace(mode="w", callback=self._change_brs_mode)
        self._change_brs_mode()
        self._current_task = None

    def _handle_classe_finalizada(self, classe):
        messagebox.showwarning(
            _("Class finished"),
            _("The class '{class_name}' was finalized").format(class_name=classe),
        )

    def goto_task(self, idx=None):
        if idx is None:
            idx = self._current_task_idx + 1
        if idx >= len(self.tasks):
            raise StopIteration()
        print("goto_task", idx, self.tasks[idx])
        self._current_task = self.tasks[idx]
        self._current_task_idx = idx
        return self._current_task

    def _goto_previous_task(self):
        if self._current_task_idx < 1:
            return
        self._current_task_idx -= 1
        self._load_image_callback(current_task=self.goto_task(self._current_task_idx))

    def _get_current_task(self, ask_next=False):
        current_class = (
            self._current_task.class_name
            if self._current_task is not None
            else _("* no one *")
        )
        try:
            if self._current_task is None or ask_next:
                logger.debug(_("Pulling next task from the iterator"))
                next_task = self.goto_task()
                if current_class != next_task.class_name:
                    self._handle_classe_finalizada(current_class)
                self._current_task = next_task
            self.task_label.config(
                text=f"class:{self._current_task.class_name} {self._current_task.image}"  # noqa: E501
            )
            return self._current_task
        except StopIteration:
            self._handle_classe_finalizada(current_class)
            exit(0)

    def _init_state(self):
        self.state = {
            "zoomin_params": {
                "use_zoom_in": tk.BooleanVar(value=True),
                "fixed_crop": tk.BooleanVar(value=True),
                "skip_clicks": tk.IntVar(value=-1),
                "target_size": tk.IntVar(value=min(400, self.limit_longest_size)),
                "expansion_ratio": tk.DoubleVar(value=1.4),
            },
            "predictor_params": {"net_clicks_limit": tk.IntVar(value=8)},
            "brs_mode": tk.StringVar(value="NoBRS"),
            "prob_thresh": tk.DoubleVar(value=0.5),
            "lbfgs_max_iters": tk.IntVar(value=20),
            "alpha_blend": tk.DoubleVar(value=0.5),
            "click_radius": tk.IntVar(value=3),
            "visualize_mask": tk.BooleanVar(value=False),
            "visualize_blender_contrast": tk.BooleanVar(value=False),
        }

    def _add_menu(self):
        self.menubar = FocusLabelFrame(self, bd=1)
        self.menubar.pack(side=tk.TOP, fill="x")

        button = FocusButton(
            self.menubar,
            text=_("Load image") + " (n)",
            command=self._load_image_callback,
        )
        button.pack(side=tk.LEFT)
        self.save_mask_btn = FocusButton(
            self.menubar,
            text=_("Save mask") + " (l)",
            command=self._save_mask_callback,
        )
        self.save_mask_btn.pack(side=tk.LEFT)
        self.save_mask_btn.configure(state=tk.DISABLED)

        self.previous_image_btn = FocusButton(
            self.menubar,
            text=_("Previous image") + " (b)",
            command=self._goto_previous_task,
        )
        self.previous_image_btn.pack(side=tk.LEFT)
        # self.previous_image_btn.configure(state=tk.DISABLED)

        self.load_mask_btn = FocusButton(
            self.menubar, text=_("Load mask"), command=self._load_mask_callback
        )
        self.load_mask_btn.pack(side=tk.LEFT)
        self.load_mask_btn.configure(state=tk.DISABLED)

        button = FocusButton(
            self.menubar, text=_("About"), command=self._about_callback
        )
        button.pack(side=tk.LEFT)
        button = FocusButton(self.menubar, text=_("Exit"), command=self.master.quit)
        button.pack(side=tk.LEFT)

        self.task_label = tk.Label(self.menubar, text=_("* status *"))
        self.task_label.pack(side=tk.LEFT)

    def _add_canvas(self):
        self.canvas_frame = FocusLabelFrame(self, text="Image")
        self.canvas_frame.rowconfigure(0, weight=1)
        self.canvas_frame.columnconfigure(0, weight=1)

        self.canvas = tk.Canvas(
            self.canvas_frame,
            highlightthickness=0,
            cursor="hand1",
            width=400,
            height=400,
        )
        self.canvas.grid(row=0, column=0, sticky="nswe", padx=5, pady=5)

        self.image_on_canvas = None
        self.canvas_frame.pack(side=tk.LEFT, fill="both", expand=True, padx=5, pady=5)

    def _add_buttons(self):
        self.control_frame = FocusLabelFrame(self, text=_("Controls"))
        self.control_frame.pack(side=tk.TOP, fill="x", padx=5, pady=5)
        master = self.control_frame

        FocusCheckButton(
            master,
            text=f"{_('Mostrar máscara ao invés da imagem')} (j)",
            variable=self.state["visualize_mask"],
        ).pack(side=tk.TOP)
        self.state["visualize_mask"].trace(
            mode="w", callback=ignore_params_then_call(self._update_image)
        )
        FocusCheckButton(
            master,
            text=f"{_('Mostrar sobreposição da máscara com mais contraste')} (k)",
            variable=self.state["visualize_blender_contrast"],
        ).pack(side=tk.TOP)
        self.state["visualize_blender_contrast"].trace(
            mode="w", callback=ignore_params_then_call(self._update_image)
        )

        self.clicks_options_frame = FocusLabelFrame(master, text=_("Clicks management"))
        self.clicks_options_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        self.finish_object_button = FocusButton(
            self.clicks_options_frame,
            text=_("Finish\nobject"),
            bg="#b6d7a8",
            fg="black",
            width=10,
            height=2,
            state=tk.DISABLED,
            command=self.controller.finish_object,
        )
        self.finish_object_button.pack(side=tk.LEFT, fill=tk.X, padx=10, pady=3)
        self.undo_click_button = FocusButton(
            self.clicks_options_frame,
            text=f"{_('Undo click')} (h)",
            bg="#ffe599",
            fg="black",
            width=10,
            height=2,
            state=tk.DISABLED,
            command=self._undo_click,
        )
        self.undo_click_button.pack(side=tk.LEFT, fill=tk.X, padx=10, pady=3)
        self.reset_clicks_button = FocusButton(
            self.clicks_options_frame,
            text=f"{_('Reset clicks')} (m)",
            bg="#ea9999",
            fg="black",
            width=10,
            height=2,
            state=tk.DISABLED,
            command=self._reset_last_object,
        )
        self.reset_clicks_button.pack(side=tk.LEFT, fill=tk.X, padx=10, pady=3)

        self.zoomin_options_frame = FocusLabelFrame(master, text=_("ZoomIn options"))
        self.zoomin_options_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        FocusCheckButton(
            self.zoomin_options_frame,
            text=_("Use ZoomIn"),
            command=self._reset_predictor,
            variable=self.state["zoomin_params"]["use_zoom_in"],
        ).grid(row=0, column=0, padx=10)
        FocusCheckButton(
            self.zoomin_options_frame,
            text=_("Fixed crop"),
            command=self._reset_predictor,
            variable=self.state["zoomin_params"]["fixed_crop"],
        ).grid(row=1, column=0, padx=10)
        tk.Label(self.zoomin_options_frame, text=_("Skip clicks")).grid(
            row=0, column=1, pady=1, sticky="e"
        )
        tk.Label(self.zoomin_options_frame, text=_("Target size")).grid(
            row=1, column=1, pady=1, sticky="e"
        )
        tk.Label(self.zoomin_options_frame, text=_("Expand ratio")).grid(
            row=2, column=1, pady=1, sticky="e"
        )
        BoundedNumericalEntry(
            self.zoomin_options_frame,
            variable=self.state["zoomin_params"]["skip_clicks"],
            min_value=-1,
            max_value=None,
            vartype=int,
            name="zoom_in_skip_clicks",
        ).grid(row=0, column=2, padx=10, pady=1, sticky="w")
        BoundedNumericalEntry(
            self.zoomin_options_frame,
            variable=self.state["zoomin_params"]["target_size"],
            min_value=100,
            max_value=self.limit_longest_size,
            vartype=int,
            name="zoom_in_target_size",
        ).grid(row=1, column=2, padx=10, pady=1, sticky="w")
        BoundedNumericalEntry(
            self.zoomin_options_frame,
            variable=self.state["zoomin_params"]["expansion_ratio"],
            min_value=1.0,
            max_value=2.0,
            vartype=float,
            name="zoom_in_expansion_ratio",
        ).grid(row=2, column=2, padx=10, pady=1, sticky="w")
        self.zoomin_options_frame.columnconfigure((0, 1, 2), weight=1)

        self.brs_options_frame = FocusLabelFrame(master, text=_("BRS options"))
        self.brs_options_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        menu = tk.OptionMenu(
            self.brs_options_frame,
            self.state["brs_mode"],
            *self.brs_modes,
            command=self._change_brs_mode,
        )
        menu.config(width=11)
        menu.grid(rowspan=2, column=0, padx=10)
        self.net_clicks_label = tk.Label(
            self.brs_options_frame, text=_("Network clicks")
        )
        self.net_clicks_label.grid(row=0, column=1, pady=2, sticky="e")
        self.net_clicks_entry = BoundedNumericalEntry(
            self.brs_options_frame,
            variable=self.state["predictor_params"]["net_clicks_limit"],
            min_value=0,
            max_value=None,
            vartype=int,
            allow_inf=True,
            name="net_clicks_limit",
        )
        self.net_clicks_entry.grid(row=0, column=2, padx=10, pady=2, sticky="w")
        self.lbfgs_iters_label = tk.Label(
            self.brs_options_frame, text=_("L-BFGS\nmax iterations")
        )
        self.lbfgs_iters_label.grid(row=1, column=1, pady=2, sticky="e")
        self.lbfgs_iters_entry = BoundedNumericalEntry(
            self.brs_options_frame,
            variable=self.state["lbfgs_max_iters"],
            min_value=1,
            max_value=1000,
            vartype=int,
            name="lbfgs_max_iters",
        )
        self.lbfgs_iters_entry.grid(row=1, column=2, padx=10, pady=2, sticky="w")
        self.brs_options_frame.columnconfigure((0, 1), weight=1)

        self.prob_thresh_frame = FocusLabelFrame(
            master, text=_("Predictions threshold")
        )
        self.prob_thresh_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        FocusHorizontalScale(
            self.prob_thresh_frame,
            from_=0.0,
            to=1.0,
            command=self._update_prob_thresh,
            variable=self.state["prob_thresh"],
        ).pack(padx=10)

        self.alpha_blend_frame = FocusLabelFrame(
            master, text=_("Alpha blending coefficient")
        )
        self.alpha_blend_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        FocusHorizontalScale(
            self.alpha_blend_frame,
            from_=0.0,
            to=1.0,
            command=self._update_blend_alpha,
            variable=self.state["alpha_blend"],
        ).pack(padx=10, anchor=tk.CENTER)

        self.click_radius_frame = FocusLabelFrame(
            master, text=_("Visualisation click radius")
        )
        self.click_radius_frame.pack(side=tk.TOP, fill=tk.X, padx=10, pady=3)
        FocusHorizontalScale(
            self.click_radius_frame,
            from_=0,
            to=7,
            resolution=1,
            command=self._update_click_radius,
            variable=self.state["click_radius"],
        ).pack(padx=10, anchor=tk.CENTER)

    def _load_image_callback(self, current_task=None):
        self.menubar.focus_set()
        if self._check_entry(self):
            if current_task is None:
                current_task = self._get_current_task(ask_next=True)
            filename = str(current_task.image.resolve())
            logger.info(_("Loading '{filename}'").format(filename=filename))
            image = cv2.cvtColor(cv2.imread(filename), cv2.COLOR_BGR2RGB)
            self.controller.set_image(image)
            points_json_name = current_task.class_name + ".json"
            points_json = current_task.output_dir / points_json_name
            if current_task.seed_mask is not None:
                points_json = current_task.seed_mask.parent / points_json_name
                if not points_json.exists():
                    points_json = current_task.output_dir / points_json_name
            if points_json.exists():
                with points_json.open("r") as f:
                    data = json.load(f)
                    for item in data:
                        self._click_callback(
                            item["is_positive"],
                            item["coords"][1],
                            item["coords"][0],
                        )
                    # from ritm_annotation.inference.clicker import Click
                    # data = [Click(item['is_positive'], item['coords'][1], item['coords'][0]) for item in data]
            else:
                if (
                    current_task.seed_mask is not None
                    and current_task.seed_mask.exists()
                ):
                    logger.debug(f"seed mask: {current_task.seed_mask}")
                    mask = cv2.imread(str(current_task.seed_mask), 0)
                    mask = mask > 127
                    self.controller.set_mask(mask)
            self.save_mask_btn.configure(state=tk.NORMAL)
            self.load_mask_btn.configure(state=tk.NORMAL)
            self._update_image()
            self._reset_predictor()

    def _undo_click(self):
        self.controller.undo_click()
        self._update_image()

    def _save_mask_callback(self):
        self.menubar.focus_set()
        if self._check_entry(self):
            mask = self.controller.result_mask
            if mask is None:
                return
            current_task = self._get_current_task()
            filename = str(current_task.mask_output.resolve())
            mask = (mask > 0) * 255
            logger.info(_("Saving '{filename}'").format(filename=filename))
            cv2.imwrite(filename, mask)
            points_json = current_task.output_dir / (current_task.class_name + ".json")
            with points_json.open("w") as f:
                # json.dump(self.controller.states, f)
                json.dump(
                    [item.to_json() for item in self.controller.clicker.get_state()],
                    f,
                )

    def _load_mask_callback(self):
        if not self.controller.net.with_prev_mask:
            messagebox.showwarning(
                _("Warning"),
                _(
                    "The current model doesn't support loading external masks. Please use ITER-M models for that purpose."
                ),
            )
            return

        self.menubar.focus_set()
        if self._check_entry(self):
            current_task = self._get_current_task()
            filename = current_task.mask_output.resolve()
            if not filename.exist() and current_task.seed_mask is not None:
                filename = current_task.seed_mask
            filename = str(filename)
            logger.debug(_("Loading '{filename}'").format(filename=filename))
            mask = cv2.imread(filename, 0)
            logger.debug(f"mask: {mask}")
            if mask is None:
                messagebox.showwarning(_("Warning"), _("No mask found"))
            else:
                mask = mask > 127
                self.controller.set_mask(mask)
                self._update_image()

    def _about_callback(self):
        self.menubar.focus_set()

        text = [
            _("Developed by:"),
            "K.Sofiiuk and I. Petrov",
            "The MIT License, 2021",
        ]

        messagebox.showinfo(_("About Demo"), "\n".join(text))

    def _reset_last_object(self):
        self.state["alpha_blend"].set(0.5)
        self.state["prob_thresh"].set(0.5)
        self.controller.reset_last_object()

    def _update_prob_thresh(self, value):
        if self.controller.is_incomplete_mask:
            self.controller.prob_thresh = self.state["prob_thresh"].get()
            self._update_image()

    def _update_blend_alpha(self, value):
        self._update_image()

    def _update_click_radius(self, *args):
        if self.image_on_canvas is None:
            return

        self._update_image()

    def _change_brs_mode(self, *args):
        if self.state["brs_mode"].get() == "NoBRS":
            self.net_clicks_entry.set("INF")
            self.net_clicks_entry.configure(state=tk.DISABLED)
            self.net_clicks_label.configure(state=tk.DISABLED)
            self.lbfgs_iters_entry.configure(state=tk.DISABLED)
            self.lbfgs_iters_label.configure(state=tk.DISABLED)
        else:
            if self.net_clicks_entry.get() == "INF":
                self.net_clicks_entry.set(8)
            self.net_clicks_entry.configure(state=tk.NORMAL)
            self.net_clicks_label.configure(state=tk.NORMAL)
            self.lbfgs_iters_entry.configure(state=tk.NORMAL)
            self.lbfgs_iters_label.configure(state=tk.NORMAL)

        self._reset_predictor()

    def _reset_predictor(self, *args, **kwargs):
        brs_mode = self.state["brs_mode"].get()
        prob_thresh = self.state["prob_thresh"].get()
        net_clicks_limit = (
            None
            if brs_mode == "NoBRS"
            else self.state["predictor_params"]["net_clicks_limit"].get()
        )

        if self.state["zoomin_params"]["use_zoom_in"].get():
            zoomin_params = {
                "skip_clicks": self.state["zoomin_params"]["skip_clicks"].get(),
                "target_size": self.state["zoomin_params"]["target_size"].get(),
                "expansion_ratio": self.state["zoomin_params"]["expansion_ratio"].get(),
            }
            if self.state["zoomin_params"]["fixed_crop"].get():
                zoomin_params["target_size"] = (
                    zoomin_params["target_size"],
                    zoomin_params["target_size"],
                )
        else:
            zoomin_params = None

        predictor_params = {
            "brs_mode": brs_mode,
            "prob_thresh": prob_thresh,
            "zoom_in_params": zoomin_params,
            "predictor_params": {
                "net_clicks_limit": net_clicks_limit,
                "max_size": self.limit_longest_size,
            },
            "brs_opt_func_params": {"min_iou_diff": 1e-3},
            "lbfgs_params": {"maxfun": self.state["lbfgs_max_iters"].get()},
        }
        self.controller.reset_predictor(predictor_params)

    def _click_callback(self, is_positive, x, y):
        self.canvas.focus_set()

        if self.image_on_canvas is None:
            messagebox.showwarning(_("Warning"), _("Please load an image first"))
            return

        if self._check_entry(self):
            self.controller.add_click(x, y, is_positive)

    def _update_image(self, reset_canvas=False):
        image = self.controller.get_visualization(
            alpha_blend=self.state["alpha_blend"].get(),
            click_radius=self.state["click_radius"].get(),
        )
        image_mask = (
            np.array(
                cv2.cvtColor(self.controller.result_mask, cv2.COLOR_GRAY2RGB),
                dtype="uint8",
            )
            * 255
        )
        if self.state["visualize_mask"].get():
            image = image_mask

        if self.state["visualize_blender_contrast"].get():
            image = (image // 2) + (image_mask // 2)

        if self.image_on_canvas is None:
            self.image_on_canvas = CanvasImage(self.canvas_frame, self.canvas)
            self.image_on_canvas.register_click_callback(self._click_callback)

        self._set_click_dependent_widgets_state()
        if image is not None:
            self.image_on_canvas.reload_image(Image.fromarray(image), reset_canvas)

    def _set_click_dependent_widgets_state(self):
        after_1st_click_state = (
            tk.NORMAL if self.controller.is_incomplete_mask else tk.DISABLED
        )
        before_1st_click_state = (
            tk.DISABLED if self.controller.is_incomplete_mask else tk.NORMAL
        )

        self.finish_object_button.configure(state=after_1st_click_state)
        self.undo_click_button.configure(state=after_1st_click_state)
        self.reset_clicks_button.configure(state=after_1st_click_state)
        self.zoomin_options_frame.set_frame_state(before_1st_click_state)
        self.brs_options_frame.set_frame_state(before_1st_click_state)

        if self.state["brs_mode"].get() == "NoBRS":
            self.net_clicks_entry.configure(state=tk.DISABLED)
            self.net_clicks_label.configure(state=tk.DISABLED)
            self.lbfgs_iters_entry.configure(state=tk.DISABLED)
            self.lbfgs_iters_label.configure(state=tk.DISABLED)

    def _check_entry(self, widget):
        all_checked = True
        if widget.winfo_children is not None:
            for w in widget.winfo_children():
                all_checked = all_checked and self._check_entry(w)

        if getattr(widget, "_check_bounds", None) is not None:
            all_checked = all_checked and widget._check_bounds(widget.get(), "-1")

        return all_checked


def handle(args):
    if not torch.cuda.is_available():
        logger.warn(
            _("CUDA support not detected. Stuff will be way slower and sluggish!")
        )
    logger.debug(_("classes: ") + ", ".join(args.classes))

    if args.device is None:
        args.device = "cuda" if torch.cuda.is_available() else "cpu"

    assert args.input.is_dir(), _("Origin should be a folder")
    # assert args.input.is_dir(), _("Origem precisa ser uma pasta")
    assert not args.output.exists() or args.output.is_dir(), _(
        "Destination should be a folder, if it isn't exist it will be created"
    )
    # ), "Destino precisa ser uma pasta, se não existe vai ser criada"

    if args.checkpoint is None:
        logger.info(_("Checkpoint not specified, using default..."))
        args.checkpoint = get_default_weight()

    torch.backends.cudnn.determistic = True

    checkpoint_path = find_checkpoint(args.checkpoint.parent, args.checkpoint.name)
    model = load_is_model(checkpoint_path, args.device, cpu_dist_maps=True)

    def _file_iterator():
        return args.input.iterdir()

    def _class_iterator():
        return iter(args.classes)

    def _fileclass_iterator():
        if args.classes_first:
            for file in _file_iterator():
                for cls in _class_iterator():
                    yield file, cls
        else:
            for cls in _class_iterator():
                for file in _file_iterator():
                    yield file, cls

    def look_for_tasks():
        for image, class_name in _fileclass_iterator():
            if not image.is_file():
                continue
            image_name = image.name
            output_dir = args.output / image_name
            output_dir.mkdir(exist_ok=True, parents=True)
            mask_output = output_dir / f"{class_name}.png"
            if mask_output.exists():
                continue
            yield edict(
                dict(
                    output_dir=output_dir,
                    image=image,
                    name=image_name,
                    mask_output=mask_output,
                    class_name=class_name,
                    seed_mask=args.seed / image_name / f"{class_name}.png"
                    if args.seed is not None
                    else None,
                )
            )

    tasks = look_for_tasks()

    root = tk.Tk()
    root.minsize(960, 480)

    app_args = edict(dict(limit_longest_size=400, device=args.device))

    app = InteractiveDemoApp(
        root, app_args, model, tasks, classes_first=args.classes_first
    )
    root.deiconify()
    app.mainloop()



import logging
import time
from gettext import gettext as _
from pathlib import Path

from ritm_annotation.utils.misc import incrf, try_tqdm

logger = logging.getLogger(__name__)

COMMAND_DESCRIPTION = _("Convert dataset in mask form to COCO")


def command(subparser):
    subparser.add_argument("input", type=Path, help=_("Dataset folder in mask form"))
    subparser.add_argument(
        "output", type=Path, help=_("Where to save the COCO dataset JSON")
    )
    subparser.add_argument(
        "--overwrite",
        action="store_true",
        help=_("Overwrite JSON file if it exists"),
    )
    subparser.add_argument(
        "--description",
        type=str,
        help=_("Description for the COCO dataset"),
        default=_("Created with ritm_annotation"),
    )

    def handle(args):
        import cv2
        import numpy as np
        import pycocotools.mask

        images_idx = incrf()
        annotations_idx = incrf()
        data = dict(
            meta=dict(
                contributor="Created with ritm_annotation",
                description=args.description,
                date_created=time.strftime("%Y/%m/%d"),
                version="1.0",
                url="https://www.youtube.com/watch?v=dQw4w9WgXcQ",
                year=time.strftime("%Y"),
            ),
            license=[
                dict(
                    id=1,
                    url="https://www.youtube.com/watch?v=dQw4w9WgXcQ",
                    name="ritm (edit later)",
                )
            ],
            images=[],
            annotations=[],
            categories=[],
        )
        assert args.input.exists() and args.input.is_dir(), _(
            "Dataset folder must exist and be a folder"
        )
        if not args.overwrite:
            assert not args.output.exists(), _(
                "COCO dataset exists, use --overwrite to ignore this"
            )
        input_items = list(args.input.iterdir())
        args.output.parent.mkdir(exist_ok=True, parents=True)

        classes = set()
        for item in try_tqdm(input_items, desc=_("Enumerating all classes...")):
            if not item.is_dir():
                continue
            for subitem in item.iterdir():
                if subitem.name.endswith(".json"):
                    continue
                classes.add(subitem.stem)  # only the part without the extension
        classes = list(classes)
        classes.sort()
        classes = {cls: i + 1 for i, cls in enumerate(classes)}
        for cls, i in classes.items():
            data["categories"].append(dict(id=i, name=cls, supercategory=None))
        for item in try_tqdm(input_items, desc=_("Ingesting images...")):
            if not item.is_dir():
                continue
            image_id = next(images_idx)
            image_to_append = dict(license=1, file_name=item.name, id=image_id)
            have_annotation = False
            for subitem in item.iterdir():
                if subitem.name.endswith(".json"):
                    continue
                segm = cv2.imread(str(subitem), 0)
                if segm is None:
                    continue
                if np.count_nonzero(segm) == 0:
                    continue
                annotation_id = next(annotations_idx)
                # segm = segm != 0
                pos = np.where(segm != 0)
                try:
                    np.min(pos[1])
                except ValueError:  # empty mask
                    continue
                (h, w) = segm.shape
                image_to_append["width"] = w
                image_to_append["height"] = h
                xmin = np.min(pos[1])
                xmax = np.max(pos[1])
                ymin = np.min(pos[0])
                ymax = np.max(pos[0])
                if abs(xmin - xmax) < 1 or abs(ymin - ymax) < 1:
                    continue  # pular máscaras vazias
                class_mask_fortran = np.asarray(segm, dtype="uint8", order="F")
                rle = pycocotools.mask.encode(class_mask_fortran)
                rle["counts"] = rle["counts"].decode("utf-8")
                data["annotations"].append(
                    dict(
                        segmentation=rle,
                        area=int(pycocotools.mask.area(rle)),
                        iscrowd=0,
                        image_id=image_id,
                        bbox=list(pycocotools.mask.toBbox(rle)),
                        category_id=classes[subitem.stem],
                        id=annotation_id,
                    )
                )
                have_annotation = True
            if have_annotation:
                data["images"].append(image_to_append)
        with args.output.open("w") as f:
            from json import dump

            dump(data, f)

    return handle



# flake8: noqa E501

from gettext import gettext as _
from pathlib import Path

COMMAND_DESCRIPTION = _("Interactively annotate a dataset")


def command(subparser):
    subparser.add_argument("input", type=Path)
    subparser.add_argument("output", type=Path)
    subparser.add_argument("-d", "--device", dest="device", default="cuda")
    subparser.add_argument("--classes-first", action="store_true")
    subparser.add_argument(
        "-c", "--classes", dest="classes", type=str, required=True, nargs="+"
    )
    subparser.add_argument("-w", "--checkpoint", dest="checkpoint", type=Path)
    subparser.add_argument(
        "-s",
        "--seed",
        dest="seed",
        type=Path,
        help=_("Folder with pre-defined segmentations to be adjusted"),
    )

    def handle(args):
        from .annotator import handle as annotator_handle

        annotator_handle(args)

    return handle



import logging
import os
import pprint
import random
from collections import defaultdict
from copy import deepcopy
from gettext import gettext as _

import cv2
import numpy as np
import torch
from torch.utils.data import DataLoader
from tqdm import tqdm

from ritm_annotation.utils.distributed import (
    get_dp_wrapper,
    get_sampler,
    reduce_loss_dict,
)
from ritm_annotation.utils.log import SummaryWriterAvg, TqdmToLogger
from ritm_annotation.utils.misc import save_checkpoint
from ritm_annotation.utils.serialization import get_config_repr
from ritm_annotation.utils.vis import draw_points, draw_probmap

from .optimizer import get_optimizer

logger = logging.getLogger(__name__)


class ISTrainer(object):
    def __init__(
        self,
        model,
        cfg,
        model_cfg,
        loss_cfg,
        trainset=None,
        valset=None,
        optimizer="adam",
        optimizer_params=None,
        image_dump_interval=200,
        checkpoint_interval=10,
        tb_dump_period=25,
        max_interactive_points=0,
        lr_scheduler=None,
        metrics=None,
        additional_val_metrics=None,
        net_inputs=("images", "points"),
        max_num_next_clicks=0,
        click_models=None,
        prev_mask_drop_prob=0.0,
        dry_run=False,
    ):
        self._ran_before_needed = False
        self.cfg = cfg
        self.model_cfg = model_cfg
        self.max_interactive_points = max_interactive_points
        self.loss_cfg = loss_cfg
        self.val_loss_cfg = deepcopy(loss_cfg)
        self.tb_dump_period = tb_dump_period
        self.net_inputs = net_inputs
        self.max_num_next_clicks = max_num_next_clicks

        self.click_models = click_models
        self.prev_mask_drop_prob = prev_mask_drop_prob

        if cfg.distributed:
            cfg.batch_size //= cfg.ngpus
            cfg.val_batch_size //= cfg.ngpus

        if metrics is None:
            metrics = []
        self.train_metrics = metrics
        self.val_metrics = deepcopy(metrics)
        if additional_val_metrics is not None:
            self.val_metrics.extend(additional_val_metrics)

        self.checkpoint_interval = checkpoint_interval
        self.image_dump_interval = image_dump_interval
        self.task_prefix = ""
        self.sw = None

        self.dry_run = dry_run
        self.trainset = trainset
        self.valset = valset
        self.device = self.cfg.device
        self.lr_scheduler_fn = lr_scheduler
        self.model = model
        self.optimizer = optimizer
        self.optimizer_params = optimizer_params

        self.tqdm_out = TqdmToLogger(logger, level=logging.INFO)

    def _before_needed_hook(self):
        if self._ran_before_needed:
            return
        self._ran_before_needed = True

        logger.debug(_("Looking for basic assumptions"))
        assert self.trainset is not None, _("Missing train dataset")
        assert self.valset is not None, _("Missing validation dataset")

        logger.info(
            _("Dataset of {num_samples} samples was loaded for training.").format(
                num_samples=self.trainset.get_samples_number()
            )  # noqa: E501
        )
        logger.info(
            _("Dataset of {num_samples} samples was loaded for validation.").format(
                num_samples=self.valset.get_samples_number()
            )  # noqa: E501
        )

        logger.debug(_("Setting up dataloder"))
        self.train_data = DataLoader(
            self.trainset,
            self.cfg.batch_size,
            sampler=get_sampler(
                self.trainset, shuffle=True, distributed=self.cfg.distributed
            )
            if not self.dry_run
            else None,
            drop_last=True,
            pin_memory=True,
            num_workers=self.cfg.workers,
            persistent_workers=self.cfg.workers > 0,
        )

        self.val_data = DataLoader(
            self.valset,
            self.cfg.val_batch_size,
            sampler=get_sampler(
                self.valset, shuffle=False, distributed=self.cfg.distributed
            )
            if not self.dry_run
            else None,
            drop_last=True,
            pin_memory=True,
            num_workers=self.cfg.workers,
            persistent_workers=self.cfg.workers > 0,
        )
        logger.debug(_("Initializing model"))
        self.model = self._load_weights(self.model)
        self.optim = get_optimizer(self.model, self.optimizer, self.optimizer_params)

        if self.cfg.multi_gpu:
            self.model = get_dp_wrapper(self.cfg.distributed)(
                self.model,
                device_ids=self.cfg.gpu_ids,
                output_device=self.cfg.gpu_ids[0],
            )

        if self.is_master:
            logger.debug(self.model)
            logger.debug(get_config_repr(self.model._config))

        self.net = self.model.to(self.device)
        self.lr = self.optimizer_params["lr"]

        if self.lr_scheduler_fn is not None:
            self.lr_scheduler = self.lr_scheduler_fn(optimizer=self.optim)
            if self.cfg.start_epoch > 0:
                for __ in range(self.cfg.start_epoch):
                    self.lr_scheduler.step()

        logger.debug(_("Initializing click models"))

        if self.click_models is not None:
            for click_model in self.click_models:
                for param in click_model.parameters():
                    param.requires_grad = False
                click_model.to(self.device)
                click_model.eval()

        logger.debug(
            _("First train batch: {batch}").format(
                batch=repr(next(iter(self.train_data)))
            )
        )
        logger.debug(
            _("First evaluation batch: {batch}").format(
                batch=repr(next(iter(self.val_data)))
            )
        )

        logger.debug(_("Run experiment with config:"))
        logger.debug(pprint.pformat(self.cfg, indent=4))

    def run(self, num_epochs, start_epoch=None, validation=True):
        self._before_needed_hook()
        if start_epoch is None:
            start_epoch = self.cfg.start_epoch

        logger.info(_("Starting Epoch: {start_epoch}").format(start_epoch=start_epoch))
        logger.info(_("Total Epochs: {num_epochs}").format(num_epochs=num_epochs))
        for epoch in range(start_epoch, num_epochs):
            self.training(epoch)
            if validation:
                self.validation(epoch)

    def training(self, epoch):
        self._before_needed_hook()
        if self.sw is None and self.is_master:
            self.sw = SummaryWriterAvg(
                log_dir=str(self.cfg.LOGS_PATH),
                flush_secs=10,
                dump_period=self.tb_dump_period,
            )

        if self.cfg.distributed:
            self.train_data.sampler.set_epoch(epoch)

        log_prefix = "Train" + self.task_prefix.capitalize()
        tbar = (
            tqdm(self.train_data, file=self.tqdm_out, ncols=100)
            if self.is_master
            else self.train_data
        )

        for metric in self.train_metrics:
            metric.reset_epoch_stats()

        self.net.train()
        train_loss = 0.0
        for i, batch_data in enumerate(tbar):
            global_step = epoch * len(self.train_data) + i

            (
                loss,
                losses_logging,
                splitted_batch_data,
                outputs,
            ) = self.batch_forward(batch_data)

            self.optim.zero_grad()
            loss.backward()
            self.optim.step()

            losses_logging["overall"] = loss
            reduce_loss_dict(losses_logging)

            train_loss += losses_logging["overall"].item()

            if self.is_master:
                for loss_name, loss_value in losses_logging.items():
                    self.sw.add_scalar(
                        tag=f"{log_prefix}Losses/{loss_name}",
                        value=loss_value.item(),
                        global_step=global_step,
                    )

                for k, v in self.loss_cfg.items():
                    if (
                        "_loss" in k
                        and hasattr(v, "log_states")
                        and self.loss_cfg.get(k + "_weight", 0.0) > 0
                    ):
                        v.log_states(self.sw, f"{log_prefix}Losses/{k}", global_step)

                if (
                    self.image_dump_interval > 0
                    and global_step % self.image_dump_interval == 0
                ):
                    self.save_visualization(
                        splitted_batch_data,
                        outputs,
                        global_step,
                        prefix="train",
                    )

                self.sw.add_scalar(
                    tag=f"{log_prefix}States/learning_rate",
                    value=self.lr
                    if not hasattr(self, "lr_scheduler")
                    else self.lr_scheduler.get_lr()[-1],
                    global_step=global_step,
                )

                tbar.set_description(
                    _("Epoch {epoch}, training loss {train_loss:.8f}").format(
                        epoch=epoch, train_loss=train_loss / (i + 1)
                    )
                )
                for metric in self.train_metrics:
                    metric.log_states(
                        self.sw,
                        f"{log_prefix}Metrics/{metric.name}",
                        global_step,
                    )

        if self.is_master:
            for metric in self.train_metrics:
                self.sw.add_scalar(
                    tag=f"{log_prefix}Metrics/{metric.name}",
                    value=metric.get_epoch_value(),
                    global_step=epoch,
                    disable_avg=True,
                )

            save_checkpoint(
                self.net,
                self.cfg.CHECKPOINTS_PATH,
                prefix=self.task_prefix,
                epoch=None,
                multi_gpu=self.cfg.multi_gpu,
            )

            if isinstance(self.checkpoint_interval, (list, tuple)):
                checkpoint_interval = [
                    x for x in self.checkpoint_interval if x[0] <= epoch
                ][-1][1]
            else:
                checkpoint_interval = self.checkpoint_interval

            if epoch % checkpoint_interval == 0:
                save_checkpoint(
                    self.net,
                    self.cfg.CHECKPOINTS_PATH,
                    prefix=self.task_prefix,
                    epoch=epoch,
                    multi_gpu=self.cfg.multi_gpu,
                )

        if hasattr(self, "lr_scheduler"):
            self.lr_scheduler.step()

    def validation(self, epoch):
        self._before_needed_hook()
        if self.sw is None and self.is_master:
            self.sw = SummaryWriterAvg(
                log_dir=str(self.cfg.LOGS_PATH),
                flush_secs=10,
                dump_period=self.tb_dump_period,
            )

        log_prefix = "Val" + self.task_prefix.capitalize()
        tbar = (
            tqdm(self.val_data, file=self.tqdm_out, ncols=100)
            if self.is_master
            else self.val_data
        )

        for metric in self.val_metrics:
            metric.reset_epoch_stats()

        val_loss = 0
        losses_logging = defaultdict(list)

        self.net.eval()
        for i, batch_data in enumerate(tbar):
            global_step = epoch * len(self.val_data) + i
            (
                loss,
                batch_losses_logging,
                splitted_batch_data,
                outputs,
            ) = self.batch_forward(batch_data, validation=True)

            batch_losses_logging["overall"] = loss
            reduce_loss_dict(batch_losses_logging)
            for loss_name, loss_value in batch_losses_logging.items():
                losses_logging[loss_name].append(loss_value.item())

            val_loss += batch_losses_logging["overall"].item()

            if self.is_master:
                tbar.set_description(
                    _("Epoch {epoch}, validation loss: {val_loss:.4f}").format(
                        epoch=epoch, val_loss=val_loss / (i + 1)
                    )
                )
                for metric in self.val_metrics:
                    metric.log_states(
                        self.sw,
                        f"{log_prefix}Metrics/{metric.name}",
                        global_step,
                    )

        if self.is_master:
            logging.debug(_("Saving epoch level losses"))
            for loss_name, loss_values in losses_logging.items():
                self.sw.add_scalar(
                    tag=f"{log_prefix}Losses/{loss_name}",
                    value=np.array(loss_values).mean(),
                    global_step=epoch,
                    disable_avg=True,
                )

            logging.debug(_("Saving epoch level metrics"))
            for metric in self.val_metrics:
                self.sw.add_scalar(
                    tag=f"{log_prefix}Metrics/{metric.name}",
                    value=metric.get_epoch_value(),
                    global_step=epoch,
                    disable_avg=True,
                )

    def batch_forward(self, batch_data, validation=False):
        self._before_needed_hook()
        metrics = self.val_metrics if validation else self.train_metrics
        losses_logging = dict()

        with torch.set_grad_enabled(not validation):
            batch_data = {k: v.to(self.device) for k, v in batch_data.items()}
            image, gt_mask, points = (
                batch_data["images"],
                batch_data["instances"],
                batch_data["points"],
            )
            # orig_image, orig_gt_mask, orig_points = (
            _, orig_gt_mask, _ = (
                image.clone(),
                gt_mask.clone(),
                points.clone(),
            )

            prev_output = torch.zeros_like(image, dtype=torch.float32)[:, :1, :, :]

            last_click_indx = None

            with torch.no_grad():
                num_iters = random.randint(0, self.max_num_next_clicks)

                for click_indx in range(num_iters):
                    last_click_indx = click_indx

                    if not validation:
                        self.net.eval()

                    if self.click_models is None or click_indx >= len(
                        self.click_models
                    ):
                        eval_model = self.net
                    else:
                        eval_model = self.click_models[click_indx]

                    net_input = (
                        torch.cat((image, prev_output), dim=1)
                        if self.net.with_prev_mask
                        else image
                    )
                    prev_output = torch.sigmoid(
                        eval_model(net_input, points)["instances"]
                    )

                    points = get_next_points(
                        prev_output, orig_gt_mask, points, click_indx + 1
                    )

                    if not validation:
                        self.net.train()

                if (
                    self.net.with_prev_mask
                    and self.prev_mask_drop_prob > 0
                    and last_click_indx is not None
                ):
                    zero_mask = (
                        np.random.random(size=prev_output.size(0))
                        < self.prev_mask_drop_prob
                    )
                    prev_output[zero_mask] = torch.zeros_like(prev_output[zero_mask])

            batch_data["points"] = points

            net_input = (
                torch.cat((image, prev_output), dim=1)
                if self.net.with_prev_mask
                else image
            )
            output = self.net(net_input, points)

            loss = 0.0
            loss = self.add_loss(
                "instance_loss",
                loss,
                losses_logging,
                validation,
                lambda: (output["instances"], batch_data["instances"]),
            )
            loss = self.add_loss(
                "instance_aux_loss",
                loss,
                losses_logging,
                validation,
                lambda: (output["instances_aux"], batch_data["instances"]),
            )

            if self.is_master:
                with torch.no_grad():
                    for m in metrics:
                        m.update(
                            *(output.get(x) for x in m.pred_outputs),
                            *(batch_data[x] for x in m.gt_outputs),
                        )
        return loss, losses_logging, batch_data, output

    def add_loss(
        self,
        loss_name,
        total_loss,
        losses_logging,
        validation,
        lambda_loss_inputs,
    ):
        loss_cfg = self.loss_cfg if not validation else self.val_loss_cfg
        loss_weight = loss_cfg.get(loss_name + "_weight", 0.0)
        if loss_weight > 0.0:
            loss_criterion = loss_cfg.get(loss_name)
            loss = loss_criterion(*lambda_loss_inputs())
            loss = torch.mean(loss)
            losses_logging[loss_name] = loss
            loss = loss_weight * loss
            total_loss = total_loss + loss

        return total_loss

    def save_visualization(self, splitted_batch_data, outputs, global_step, prefix):
        output_images_path = self.cfg.VIS_PATH / prefix
        if self.task_prefix:
            output_images_path /= self.task_prefix

        if not output_images_path.exists():
            output_images_path.mkdir(parents=True)
        image_name_prefix = f"{global_step:06d}"

        def _save_image(suffix, image):
            cv2.imwrite(
                str(output_images_path / f"{image_name_prefix}_{suffix}.jpg"),
                image,
                [cv2.IMWRITE_JPEG_QUALITY, 85],
            )

        images = splitted_batch_data["images"]
        points = splitted_batch_data["points"]
        instance_masks = splitted_batch_data["instances"]

        gt_instance_masks = instance_masks.cpu().numpy()
        predicted_instance_masks = (
            torch.sigmoid(outputs["instances"]).detach().cpu().numpy()
        )
        points = points.detach().cpu().numpy()

        image_blob, points = images[0], points[0]
        gt_mask = np.squeeze(gt_instance_masks[0], axis=0)
        predicted_mask = np.squeeze(predicted_instance_masks[0], axis=0)

        image = image_blob.cpu().numpy() * 255
        image = image.transpose((1, 2, 0))

        image_with_points = draw_points(
            image, points[: self.max_interactive_points], (0, 255, 0)
        )
        image_with_points = draw_points(
            image_with_points,
            points[self.max_interactive_points :],
            (0, 0, 255),
        )

        gt_mask[gt_mask < 0] = 0.25
        gt_mask = draw_probmap(gt_mask)
        predicted_mask = draw_probmap(predicted_mask)
        viz_image = np.hstack((image_with_points, gt_mask, predicted_mask)).astype(
            np.uint8
        )

        _save_image("instance_segmentation", viz_image[:, :, ::-1])

    def _load_weights(self, net):
        if self.cfg.resume_exp is not None:
            checkpoint_glob = f"{self.cfg.resume_prefix}*.pth"
            checkpoints = list(self.cfg.CHECKPOINTS_PATH.glob(checkpoint_glob))
            assert len(checkpoints) == 1, _(
                "'{checkpoint_glob}' didn't match anything"
            ).format(checkpoint_glob=f"{self.cfg.CHECKPOINTS_PATH}/{checkpoint_glob}")

            checkpoint_path = checkpoints[0]
            load_weights(net, str(checkpoint_path))
        elif self.cfg.weights is not None:
            if os.path.isfile(self.cfg.weights):
                load_weights(net, self.cfg.weights)
                self.cfg.weights = None
            else:
                raise RuntimeError(
                    _("=> no checkpoint found at '{weights_dir}'").format(
                        weights_dir=self.cfg.weights
                    )
                )

        return net

    @property
    def is_master(self):
        return self.cfg.local_rank == 0


def get_next_points(pred, gt, points, click_indx, pred_thresh=0.49):
    assert click_indx > 0
    pred = pred.cpu().numpy()[:, 0, :, :]
    gt = gt.cpu().numpy()[:, 0, :, :] > 0.5

    fn_mask = np.logical_and(gt, pred < pred_thresh)
    fp_mask = np.logical_and(np.logical_not(gt), pred > pred_thresh)

    fn_mask = np.pad(fn_mask, ((0, 0), (1, 1), (1, 1)), "constant").astype(np.uint8)
    fp_mask = np.pad(fp_mask, ((0, 0), (1, 1), (1, 1)), "constant").astype(np.uint8)
    num_points = points.size(1) // 2
    points = points.clone()

    for bindx in range(fn_mask.shape[0]):
        fn_mask_dt = cv2.distanceTransform(fn_mask[bindx], cv2.DIST_L2, 5)[1:-1, 1:-1]
        fp_mask_dt = cv2.distanceTransform(fp_mask[bindx], cv2.DIST_L2, 5)[1:-1, 1:-1]

        fn_max_dist = np.max(fn_mask_dt)
        fp_max_dist = np.max(fp_mask_dt)

        is_positive = fn_max_dist > fp_max_dist
        dt = fn_mask_dt if is_positive else fp_mask_dt
        inner_mask = dt > max(fn_max_dist, fp_max_dist) / 2.0
        indices = np.argwhere(inner_mask)
        if len(indices) > 0:
            coords = indices[np.random.randint(0, len(indices))]
            if is_positive:
                points[bindx, num_points - click_indx, 0] = float(coords[0])
                points[bindx, num_points - click_indx, 1] = float(coords[1])
                points[bindx, num_points - click_indx, 2] = float(click_indx)
            else:
                points[bindx, 2 * num_points - click_indx, 0] = float(coords[0])
                points[bindx, 2 * num_points - click_indx, 1] = float(coords[1])
                points[bindx, 2 * num_points - click_indx, 2] = float(click_indx)

    return points


def load_weights(model, path_to_weights):
    logger.info(
        _("Loading weights from path: '{path_to_weights}'").format(
            path_to_weights=path_to_weights
        )
    )
    current_state_dict = model.state_dict()
    new_state_dict = torch.load(path_to_weights, map_location="cpu")["state_dict"]
    current_state_dict.update(new_state_dict)
    model.load_state_dict(current_state_dict)



from .grabcut import GrabCutDataset


class BerkeleyDataset(GrabCutDataset):
    def __init__(self, dataset_path, **kwargs):
        super().__init__(
            dataset_path, images_dir_name="images", masks_dir_name="masks", **kwargs
        )



from gettext import gettext as _
from pathlib import Path

COMMAND_DESCRIPTION = _("Show information about a model pth file")


def command(subparser):
    subparser.add_argument("model", type=Path)

    def handle(args):
        from .model_info import handle as model_info_handle

        model_info_handle(args)

    return handle



from gettext import gettext as _
from pathlib import Path

COMMAND_DESCRIPTION = _("Run from-scratch trains using off the shelf datasets")


def command(parser):
    parser.add_argument("model_path", type=str, help=_("Path to the model script."))

    parser.add_argument(
        "-o",
        "--output",
        dest="experiment_path",
        type=Path,
        default=Path("."),
        help=_("Where to store experiment data"),
    )

    parser.add_argument(
        "-n",
        "--num-epochs",
        dest="num_epochs",
        type=int,
        help=_("Amount of epochs"),
    )

    parser.add_argument(
        "--exp-name",
        type=str,
        default="",
        help=_(
            "Here you can specify the name of the experiment. It will be added as a suffix to the experiment folder."
        ),
    )  # noqa:E501

    parser.add_argument(
        "--workers",
        type=int,
        default=4,
        metavar="N",
        help=_("Dataloader threads."),
    )

    parser.add_argument(
        "--batch-size",
        "-b",
        dest="batch_size",
        type=int,
        default=-1,
        help=_("You can override model batch size by specify positive number."),
    )  # noqa:E501

    parser.add_argument(
        "--ngpus",
        type=int,
        default=1,
        help=_(
            'Number of GPUs. If you only specify "--gpus" argument, the ngpus value will be calculated automatically. You should use either this argument or "--gpus".'
        ),
    )  # noqa:E501

    parser.add_argument(
        "--gpus",
        type=str,
        default="",
        required=False,
        help=_('Ids of used GPUs. You should use either this argument or "--ngpus".'),  # noqa: E501
    )  # noqa:E501

    parser.add_argument(
        "--resume-exp",
        type=str,
        default=None,
        help=_(
            'The prefix of the name of the experiment to be continued. If you use this field, you must specify the "--resume-prefix" argument.'
        ),  # noqa:E501
    )  # noqa:E501

    parser.add_argument(
        "--resume-prefix",
        type=str,
        default="latest",
        help=_("The prefix of the name of the checkpoint to be loaded."),
    )  # noqa:E501

    parser.add_argument(
        "--start-epoch",
        type=int,
        default=0,
        help=_(
            "The number of the starting epoch from which training will continue. (it is important for correct logging and learning rate)"
        ),
    )  # noqa:E501

    parser.add_argument(
        "--weights",
        type=str,
        default=None,
        help=_(
            "Model weights will be loaded from the specified path if you use this argument."
        ),  # noqa: E501
    )  # noqa:E501

    parser.add_argument(
        "--temp-model-path",
        type=str,
        default="",
        help=_("Do not use this argument (for internal purposes)."),
    )  # noqa:E501

    parser.add_argument("--local_rank", type=int, default=0)

    def handle(args):
        from .train import handle as train_handle

        train_handle(args)

    return handle
